@@ -8,6 +8,7 @@ from .database import insert
 from .database import make_tables
 from .database import select
 from .database import select_all
+from .database import select_multi
 from .database import tiered_path
 from .database import update
 from .database import vacuum
@@ -18,6 +19,8 @@ from .journals import journals_table
 from .journals import remove_journal
 from .journals import save_journal
 from .journals import search_journals
+from .settings import add_history
+from .settings import read_history
 from .settings import read_setting
 from .settings import settings_table
 from .settings import write_setting
@@ -34,6 +37,9 @@ from .users import edit_user_field_remove
 from .users import edit_user_field_replace
 from .users import exist_user
 from .users import exist_user_field_value
+from .users import find_user_from_fields
+from .users import find_user_from_journal
+from .users import find_user_from_submission
 from .users import new_user
 from .users import remove_user
 from .users import users_fields
