@@ -20,8 +20,13 @@ from .database import select
 from .database import select_all
 from .database import tiered_path
 from .database import update
+from .journals import journals_table
+from .settings import add_history
 from .settings import read_setting
+from .settings import settings_table
 from .settings import write_setting
+from .submissions import submissions_table
+from .users import users_table
 
 
 def get_version(db: Connection) -> str:
@@ -155,6 +160,66 @@ def make_database_3_1(db: Connection):
     db.commit()
 
 
+def make_database_3_2(db: Connection) -> Connection:
+    db.execute(
+        f"""CREATE TABLE IF NOT EXISTS USERS
+        (USERNAME TEXT UNIQUE NOT NULL,
+        FOLDERS TEXT NOT NULL,
+        GALLERY TEXT,
+        SCRAPS TEXT,
+        FAVORITES TEXT,
+        MENTIONS TEXT,
+        JOURNALS TEXT,
+        PRIMARY KEY (USERNAME ASC));"""
+    )
+
+    db.execute(
+        f"""CREATE TABLE IF NOT EXISTS SUBMISSIONS
+        (ID INT UNIQUE NOT NULL,
+        AUTHOR TEXT NOT NULL,
+        TITLE TEXT,
+        UDATE DATE NOT NULL,
+        DESCRIPTION TEXT,
+        TAGS TEXT,
+        CATEGORY TEXT,
+        SPECIES TEXT,
+        GENDER TEXT,
+        RATING TEXT,
+        FILELINK TEXT,
+        FILEEXT TEXT,
+        FILESAVED INT,
+        PRIMARY KEY (ID ASC));"""
+    )
+
+    db.execute(
+        f"""CREATE TABLE IF NOT EXISTS JOURNALS
+        (ID INT UNIQUE NOT NULL,
+        AUTHOR TEXT NOT NULL,
+        TITLE TEXT,
+        UDATE DATE NOT NULL,
+        CONTENT TEXT,
+        PRIMARY KEY (ID ASC));"""
+    )
+
+    db.execute(
+        f"""CREATE TABLE IF NOT EXISTS SETTINGS
+        (SETTING TEXT UNIQUE,
+        SVALUE TEXT,
+        PRIMARY KEY (SETTING ASC));"""
+    )
+
+    insert(db, settings_table, ["SETTING", "SVALUE"], ["USRN", "0"], False)
+    insert(db, settings_table, ["SETTING", "SVALUE"], ["SUBN", "0"], False)
+    insert(db, settings_table, ["SETTING", "SVALUE"], ["JRNN", "0"], False)
+    insert(db, settings_table, ["SETTING", "SVALUE"], ["LASTUPDATE", "0"], False)
+    insert(db, settings_table, ["SETTING", "SVALUE"], ["LASTSTART", "0"], False)
+    insert(db, settings_table, ["SETTING", "SVALUE"], ["COOKIES", "{}"], False)
+    insert(db, settings_table, ["SETTING", "SVALUE"], ["FILESFOLDER", "FA.files"], False)
+    insert(db, settings_table, ["SETTING", "SVALUE"], ["VERSION", __version__], False)
+
+    return db
+
+
 def update_2_7_to_3(db: Connection) -> Connection:
     print("Updating 2.7.0 to 3.0.0")
     db_new: Optional[Connection] = None
@@ -339,7 +404,7 @@ def update_3_1_to_3_2(db: Connection) -> Connection:
 
     try:
         db_new = connect_database("FA_new.db")
-        make_tables(db_new)
+        make_database_3_2(db_new)
 
         # Transfer common submissions and users data
         print("Transfer common submissions and users data")
@@ -382,7 +447,65 @@ def update_3_1_to_3_2(db: Connection) -> Connection:
     return connect_database("FA.db")
 
 
-def update_to_current(db: Connection) -> Connection:
+def update_3_2_to_3_3(db: Connection) -> Connection:
+    print("Updating 3.2.0 to 3.3.0")
+    db_new: Optional[Connection] = None
+    db_open: bool = True
+
+    try:
+        db_new = connect_database("FA_new.db")
+        make_tables(db_new)
+
+        # Transfer common submissions and users data
+        print("Transfer common submissions and users data")
+        db.execute("ATTACH DATABASE 'FA_new.db' AS db_new")
+        db.execute(
+            f"""INSERT OR IGNORE INTO db_new.{users_table}
+            SELECT * FROM {users_table}"""
+        )
+        db.execute(
+            f"""INSERT OR IGNORE INTO db_new.{submissions_table}
+            SELECT * FROM {submissions_table}"""
+        )
+        db.execute(
+            f"""INSERT OR IGNORE INTO db_new.{journals_table}
+            SELECT * FROM {journals_table}"""
+        )
+        db.execute(
+            f"""INSERT OR IGNORE INTO db_new.{settings_table}
+            SELECT * FROM {settings_table} WHERE SETTING != "LASTSTART" AND SETTING != "LASTUPDATE";"""
+        )
+
+        # Add update to history
+        last_update: str = select(db, settings_table, ["SVALUE"], "SETTING", "LASTUPDATE").fetchone()
+        if last_update and last_update[0] != "0":
+            # Commit and close databases to unlock
+            db.commit()
+            db.close()
+            db = None
+            add_history(db_new, float(last_update[0]), "update")
+
+        # Close databases and replace old database
+        print("Close databases and replace old database")
+        db_new.commit()
+        db_new.close()
+        move("FA.db", "FA_3.db")
+        move("FA_new.db", "FA.db")
+    except (BaseException, Exception) as err:
+        print("Database update interrupted!")
+        if db is not None:
+            db.commit()
+            db.close()
+        if db_new is not None:
+            db_new.commit()
+            db_new.close()
+        raise err
+
+    return connect_database("FA.db")
+
+
+def update_to_current(db: Connection, version: str) -> Connection:
+    print(f"Updating {version} to {__version__}")
     write_setting(db, "VERSION", __version__)
     return db
 
@@ -402,7 +525,9 @@ def update_database(db: Connection) -> Connection:
         return update_database(update_3_to_3_1(db))
     elif v >= 0 and (v := compare_versions(db_version, "3.2.0")) < 0:
         return update_database(update_3_1_to_3_2(db))
-    elif v and compare_versions(db_version, __version__) < 0:
-        return update_to_current(db)
+    elif v >= 0 and (v := compare_versions(db_version, "3.3.0")) < 0:
+        return update_database(update_3_2_to_3_3(db))
+    elif v >= 0 and compare_versions(db_version, __version__) < 0:
+        return update_to_current(db, db_version)
 
     return db
