@@ -4,9 +4,11 @@ from typing import Optional
 from typing import Tuple
 
 from .database import Connection
+from .database import Cursor
 from .database import delete
 from .database import insert
 from .database import select
+from .database import select_multi
 from .database import update
 
 """
@@ -79,8 +81,8 @@ def edit_user_field_add(db: Connection, user: str, field: str, values: List[str]
     if old_values_raw is None:
         return
 
-    values = old_values_raw[0].split(",") + values
-    values.sort()
+    old_values: List[str] = old_values_raw[0].split(",") if old_values_raw[0] else []
+    values = list(set(old_values + values))
 
     edit_user_field_replace(db, user, [field], [",".join(values)])
 
@@ -90,6 +92,19 @@ def edit_user_field_remove(db: Connection, user: str, field: str, values: List[s
     if old_values_raw is None:
         return
 
-    values = [v for v in old_values_raw[0].split(",") if v not in values]
+    old_values: List[str] = old_values_raw[0].split(",") if old_values_raw[0] else []
+    values = [v for v in old_values if v not in values]
 
     edit_user_field_replace(db, user, [field], [",".join(values)])
+
+
+def find_user_from_fields(db: Connection, fields: List[str], values: List[str], and_: bool = False) -> Cursor:
+    return select_multi(db, users_table, ["USERNAME"], fields, values, True, and_)
+
+
+def find_user_from_submission(db: Connection, submission_id: int) -> Cursor:
+    return find_user_from_fields(db, ["GALLERY", "SCRAPS", "FAVORITES", "MENTIONS"], [f"%{int(submission_id):010}%"])
+
+
+def find_user_from_journal(db: Connection, journal_id: int) -> Cursor:
+    return find_user_from_fields(db, ["JOURNALS"], [f"%{int(journal_id):010}%"])
